@@ -1,10 +1,9 @@
 MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(),
-                       lrPars = list(), DERIV)
+                       lrPars = list(), lr.random = list(), DERIV)
 {
     if(is.null(random)) random <- list()
     itemtype <- sapply(pars[[1]], class)
     has_graded <- any(itemtype == 'graded')
-    lr.random <- list() #FIXME overwrite later
     RAND <- length(random) > 0L; LR.RAND <- length(lr.random) > 0L
     RANDSTART <- list$RANDSTART
     LRPARS <- length(lrPars) > 0L
@@ -45,6 +44,12 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         nfullpars <- nfullpars + length(lrPars@par)
         estpars <- c(estpars, lrPars@est)
     }
+    if(LR.RAND){
+        for(i in 1L:length(lr.random)){
+            nfullpars <- nfullpars + length(lr.random[[i]]@par)
+            estpars <- c(estpars, lr.random[[i]]@est)
+        }
+    }
     index <- 1L:nfullpars
     N <- nrow(gtheta0[[1L]])
     #Burn in
@@ -94,6 +99,10 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         }
         if(LRPARS)
             longpars[lrPars@parnum] <- lrPars@par
+        if(LR.RAND){
+            for(i in 1L:length(lr.random))
+                longpars[lr.random[[i]]@parnum] <- lr.random[[i]]@par
+        }
     }
     names(longpars) <- names(estpars)
     stagecycle <- 1L
@@ -129,6 +138,12 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         LBOUND <- c(LBOUND, lrPars@lbound)
         UBOUND <- c(UBOUND, lrPars@ubound)
     }
+    if(LR.RAND){
+        for(i in 1L:length(lr.random)){
+            LBOUND <- c(LBOUND, lr.random[[i]]@lbound)
+            UBOUND <- c(UBOUND, lr.random[[i]]@ubound)
+        }
+    }
     for(g in 1L:ngroups)
         for(i in 1L:J)
             pars[[g]][[i]]@dat <- Data$fulldata[[g]][, c(itemloc[i]:(itemloc[i+1L] - 1L))]
@@ -154,7 +169,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             gamma <- 0
         }
         #Reload pars list
-        if(list$USEEM) longpars <- list$startlongpars
+        if(list$SE) longpars <- list$startlongpars
         pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
         if(has_graded){
             for(g in 1L:length(pars)){
@@ -175,42 +190,90 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             lrPars@mus <- lrPars@X %*% lrPars@beta
             gstructgrouppars[[1L]]$gmeans <- lrPars@mus
         }
-        if(RAND && cycles > RANDSTART) random <- reloadRandom(random=random, longpars=longpars,
-                                        parstart=max(pars[[1L]][[J+1L]]@parnum) + 1L)
+        if(LR.RAND && cycles > RANDSTART){
+            for(j in 1L:length(lr.random))
+                gstructgrouppars[[1L]]$gmeans <- gstructgrouppars[[1L]]$gmeans +
+                    lr.random[[j]]@drawvals[lr.random[[j]]@mtch]
+        }
+        if(RAND && cycles > RANDSTART) random <- reloadRandom(random=random, longpars=longpars)
+        if(LR.RAND && cycles > RANDSTART) lr.random <- reloadRandom(random=lr.random, longpars=longpars)
 
         start <- proc.time()[3L]
-        if(RAND && cycles == RANDSTART){
+        if((RAND || LR.RAND) && cycles == RANDSTART){
             gtheta0[[1L]] <- matrix(0, nrow(gtheta0[[1L]]), ncol(gtheta0[[1L]]))
-            OffTerm <- OffTerm(random, J=J, N=N)
-            if(!is.null(list$cand.t.var))
-                for(j in 1L:length(random)) random[[j]]@cand.t.var <- list$cand.t.var[j + 1L]
-            for(j in 1L:length(random)){
-                tmp <- .1
-                for(i in 1L:31L){
-                    random[[j]]@drawvals <- DrawValues(random[[j]], itemloc=itemloc,
-                                                       Theta=gtheta0[[1L]],
-                                                       pars=pars[[1L]], fulldata=Data$fulldata[[1L]],
-                                                       offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
-                    OffTerm <- OffTerm(random, J=J, N=N)
-                    if(is.null(list$cand.t.var)){
-                        if(i > 5L){
-                            if(attr(random[[j]]@drawvals,"Proportion Accepted") > .4)
-                                random[[j]]@cand.t.var <- random[[j]]@cand.t.var + 2*tmp
-                            if(attr(random[[j]]@drawvals,"Proportion Accepted") < .2)
-                                random[[j]]@cand.t.var <- random[[j]]@cand.t.var - 2*tmp
-                            if(attr(random[[j]]@drawvals,"Proportion Accepted") < .05)
-                                random[[j]]@cand.t.var <- random[[j]]@cand.t.var - 5*tmp
-                            if (random[[j]]@cand.t.var < 0){
-                                random[[j]]@cand.t.var <- tmp
-                                tmp <- tmp / 10
+            if(RAND){
+                OffTerm <- OffTerm(random, J=J, N=N)
+                if(!is.null(list$cand.t.var))
+                    for(j in 1L:length(random)) random[[j]]@cand.t.var <- list$cand.t.var[j + 1L]
+                for(j in 1L:length(random)){
+                    tmp <- .1
+                    for(i in 1L:31L){
+                        random[[j]]@drawvals <- DrawValues(random[[j]], itemloc=itemloc,
+                                                           Theta=gtheta0[[1L]],
+                                                           pars=pars[[1L]], fulldata=Data$fulldata[[1L]],
+                                                           offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
+                        OffTerm <- OffTerm(random, J=J, N=N)
+                        if(is.null(list$cand.t.var)){
+                            if(i > 5L){
+                                if(attr(random[[j]]@drawvals,"Proportion Accepted") > .4)
+                                    random[[j]]@cand.t.var <- random[[j]]@cand.t.var + 2*tmp
+                                if(attr(random[[j]]@drawvals,"Proportion Accepted") < .2)
+                                    random[[j]]@cand.t.var <- random[[j]]@cand.t.var - 2*tmp
+                                if(attr(random[[j]]@drawvals,"Proportion Accepted") < .05)
+                                    random[[j]]@cand.t.var <- random[[j]]@cand.t.var - 5*tmp
+                                if (random[[j]]@cand.t.var < 0){
+                                    random[[j]]@cand.t.var <- tmp
+                                    tmp <- tmp / 10
+                                }
                             }
                         }
                     }
+                    #better start values
+                    tmp <- nrow(random[[j]]@drawvals)
+                    tmp <- cov(random[[j]]@drawvals) * (tmp / (tmp-1L))
+                    random[[j]]@par[random[[j]]@est] <- tmp[lower.tri(tmp, TRUE)][random[[j]]@est]
+                }
+            }
+            if(LR.RAND){
+                if(!is.null(list$cand.t.var)){
+                    for(j in 1L:length(lr.random)) lr.random[[j]]@cand.t.var <-
+                            list$cand.t.var[j + length(random) + 1L]
+                }
+                for(j in 1L:length(lr.random)){
+                    tmp <- .1
+                    for(i in 1L:31L){
+                        lr.random[[j]]@drawvals <- DrawValues(lr.random[[j]], itemloc=itemloc,
+                                                           Theta=gtheta0[[1L]], LR=TRUE,
+                                                           pars=pars[[1L]], fulldata=Data$fulldata[[1L]],
+                                                           offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND)
+                        if(is.null(list$cand.t.var)){
+                            if(i > 5L){
+                                if(attr(lr.random[[j]]@drawvals,"Proportion Accepted") > .4)
+                                    lr.random[[j]]@cand.t.var <- lr.random[[j]]@cand.t.var + 2*tmp
+                                if(attr(lr.random[[j]]@drawvals,"Proportion Accepted") < .2)
+                                    lr.random[[j]]@cand.t.var <- lr.random[[j]]@cand.t.var - 2*tmp
+                                if(attr(lr.random[[j]]@drawvals,"Proportion Accepted") < .05)
+                                    lr.random[[j]]@cand.t.var <- lr.random[[j]]@cand.t.var - 5*tmp
+                                if (lr.random[[j]]@cand.t.var < 0){
+                                    lr.random[[j]]@cand.t.var <- tmp
+                                    tmp <- tmp / 10
+                                }
+                            }
+                        }
+                    }
+                    #better start values
+                    tmp <- nrow(lr.random[[j]]@drawvals)
+                    tmp <- cov(lr.random[[j]]@drawvals) * (tmp / (tmp-1L))
+                    lr.random[[j]]@par[lr.random[[j]]@est] <-
+                        tmp[lower.tri(tmp, TRUE)][lr.random[[j]]@est]
+                    for(j in 1L:length(lr.random))
+                        gstructgrouppars[[1L]]$gmeans <- gstructgrouppars[[1L]]$gmeans +
+                            lr.random[[j]]@drawvals[lr.random[[j]]@mtch]
+                    gstructgrouppars[[1L]]$gcov <- cov(gtheta0[[1L]] - gstructgrouppars[[1L]]$gmeans)
+                    tmp <- c(numeric(nfact),
+                             as.vector(gstructgrouppars[[1L]]$gcov[lower.tri(gstructgrouppars[[1L]]$gcov, TRUE)]))
+                    pars[[1L]][[J+1L]]@par[pars[[1L]][[J+1L]]@est] <- tmp[pars[[1L]][[J+1L]]@est]
                 }
-                #better start values
-                tmp <- nrow(random[[j]]@drawvals)
-                tmp <- cov(random[[j]]@drawvals) * (tmp / (tmp-1L))
-                random[[j]]@par[random[[j]]@est] <- tmp[lower.tri(tmp, TRUE)][random[[j]]@est]
             }
             cand.t.var <- if(is.null(list$cand.t.var)) .5 else list$cand.t.var[1L]
             tmp <- .1
@@ -242,11 +305,6 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 tmp2[pars[[1L]][[length(pars[[1L]])]]@est]
         }
 
-        # deal with latent regression random effects TODO
-        if(LR.RAND){
-            browser()
-        }
-
         #Step 1. Generate m_k datasets of theta
         LL <- 0
         for(g in 1L:ngroups){
@@ -267,6 +325,16 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 }
             }
         }
+        if(LR.RAND && cycles > RANDSTART){
+            for(j in 1L:length(lr.random)){
+                for(i in 1L:MHDRAWS){
+                    lr.random[[j]]@drawvals <- DrawValues(lr.random[[j]], Theta=gtheta0[[1L]],
+                                                          itemloc=itemloc, pars=pars[[1L]],
+                                                          fulldata=Data$fulldata[[1L]],
+                                                          offterm0=OffTerm, CUSTOM.IND=CUSTOM.IND, LR=TRUE)
+                }
+            }
+        }
         #adjust cand.t.var
         PA <- sapply(gtheta0, function(x) attr(x, "Proportion Accepted"))
         NS <- sapply(gtheta0, function(x) nrow(x))
@@ -279,6 +347,13 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                                    random[[j]]@cand.t.var, min = .01, max = .5)
                 }
             }
+            if(LR.RAND && cycles > (RANDSTART + 1L)){
+                for(j in 1L:length(lr.random)){
+                    lr.random[[j]]@cand.t.var <- controlCandVar(
+                        attr(lr.random[[j]]@drawvals, "Proportion Accepted"),
+                        lr.random[[j]]@cand.t.var, min = .01, max = .5)
+                }
+            }
         }
         Draws.time <- Draws.time + proc.time()[3L] - start
 
@@ -289,7 +364,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             gthetatmp <- lapply(gtheta0, function(x, prodlist) prodterms(x, prodlist),
                               prodlist=prodlist)
         tmp <- .Call('computeDPars', pars, gthetatmp, OffTerm, length(longpars), TRUE,
-                     USE.FIXED, 0L)
+                     USE.FIXED, 0L, FALSE)
         g <- tmp$grad; h <- tmp$hess
         if(length(list$SLOW.IND)){
             for(group in 1L:ngroups){
@@ -314,7 +389,7 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             if(cycles <= RANDSTART){
                 for(i in 1L:length(random)){
                     g[random[[i]]@parnum] <- 0
-                    h[random[[i]]@parnum, random[[i]]@parnum] <- diag(length(random[[i]]@parnum))
+                    h[random[[i]]@parnum, random[[i]]@parnum] <- -diag(length(random[[i]]@parnum))
                 }
             } else {
                 for(i in 1L:length(random)){
@@ -332,6 +407,20 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                   lrPars@parnum[1L:nrow(deriv$grad) + nrow(deriv$grad)*i]] <- deriv$hess
 
         }
+        if(LR.RAND){
+            if(cycles <= RANDSTART){
+                for(i in 1L:length(lr.random)){
+                    g[lr.random[[i]]@parnum] <- 0
+                    h[lr.random[[i]]@parnum, lr.random[[i]]@parnum] <- -diag(length(lr.random[[i]]@parnum))
+                }
+            } else {
+                for(i in 1L:length(lr.random)){
+                    deriv <- RandomDeriv(x=lr.random[[i]])
+                    g[lr.random[[i]]@parnum] <- deriv$grad
+                    h[lr.random[[i]]@parnum, lr.random[[i]]@parnum] <- deriv$hess
+                }
+            }
+        }
         if(length(constrain)){
             grad <- as.numeric(updateGrad(g, L))
             ave.h <- updateHess(-h, L)
@@ -341,9 +430,10 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         }
         grad <- grad[estpars & !redun_constr]
         ave.h <- ave.h[estpars & !redun_constr, estpars & !redun_constr]
-        if(any(is.na(grad)))
+        if(any(is.na(grad))){
             stop('Model did not converge (unacceptable gradient caused by extreme parameter values)',
                  call.=FALSE)
+        }
         if(is.na(attr(gtheta0[[1L]],"log.lik")))
             stop('Estimation halted. Model did not converge.', call.=FALSE)
         if(verbose){
@@ -355,6 +445,12 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
                 CTV <- c(CTV, do.call(c, lapply(random,
                                                 function(x) x@cand.t.var)))
             }
+            if(LR.RAND && cycles > RANDSTART){
+                AR <- c(AR, do.call(c, lapply(lr.random,
+                                              function(x) attr(x@drawvals, "Proportion Accepted"))))
+                CTV <- c(CTV, do.call(c, lapply(lr.random,
+                                                function(x) x@cand.t.var)))
+            }
             AR <- paste0(sapply(AR, function(x) sprintf('%.2f', x)), collapse='; ')
             CTV <- paste0(sapply(CTV, function(x) sprintf('%.2f', x)), collapse='; ')
             if(cycles <= BURNIN)
@@ -412,35 +508,45 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
         if(all(abs(gamma*correction) < TOL)) conv <- conv + 1L
         else conv <- 0L
         if(!list$SE && conv >= 3L) break
-        if(list$SE && cycles >= (400L + BURNIN + SEMCYCLES) && conv >= 3L) break
+        if(list$SE.type == 'MHRM' && list$SE &&
+           cycles >= (400L + BURNIN + SEMCYCLES) && conv >= 3L) break
         #Extra: Approximate information matrix.	sqrt(diag(solve(info))) == SE
         if(cycles == (BURNIN + SEMCYCLES + 1L)){
-            phi <- grad
-            Phi <- ave.h
+            if(list$SE.type == 'MHRM'){
+                phi <- grad
+                Phi <- ave.h
+            } else Phi <- 0
+        }
+        if(list$SE.type != 'none' && list$SE){
+            if(list$SE.type == 'MHRM'){
+                phi <- phi + gamma*(grad - phi)
+                Phi <- Phi + gamma*(ave.h - outer(grad,grad) - Phi)
+            } else if(list$SE.type == 'FMHRM'){
+                Phi <- Phi + 1/NCYCLES * (ave.h - outer(grad,grad))
+            }
         }
-        phi <- phi + gamma*(grad - phi)
-        Phi <- Phi + gamma*(ave.h - outer(grad,grad) - Phi)
         Mstep.time <- Mstep.time + proc.time()[3L] - start
     } ###END BIG LOOP
     if(verbose) cat('\r\n')
-    info <- Phi + outer(phi,phi)
     #Reload final pars list
-    if(cycles == NCYCLES + BURNIN + SEMCYCLES && !list$USEEM){
+    if(cycles == NCYCLES + BURNIN + SEMCYCLES && !list$SE){
         if(list$message)
             message('MHRM terminated after ', NCYCLES, ' iterations.')
         converge <- FALSE
     }
-    if(list$USEEM) longpars <- list$startlongpars
+    if(list$SE) longpars <- list$startlongpars
     for(g in 1L:ngroups){
         for(i in 1L:(J+1L))
             pars[[g]][[i]]@par <- longpars[pars[[g]][[i]]@parnum]
     }
     fail_invert_info <- TRUE
     if(list$SE){
+        if(list$SE.type == 'MHRM') info <- Phi + outer(phi,phi)
+        else if(list$SE.type == 'FMHRM') info <- Phi
         acov <- try(solve(info), TRUE)
         if(is(acov, 'try-error')){
             if(list$warn)
-                warning('Could not invert information matrix; model likely is not identified.',
+                warning('Could not invert information matrix; may not be identified.',
                         call.=FALSE)
         } else {
             fail_invert_info <- FALSE
@@ -464,14 +570,19 @@ MHRM.group <- function(pars, constrain, Ls, Data, PrepList, list, random = list(
             if(LRPARS){
                 lrPars@SEpar <- SE[lrPars@parnum]
             }
+            if(LR.RAND){
+                for(i in 1L:length(lr.random))
+                    lr.random[[i]]@SEpar <- SE[lr.random[[i]]@parnum]
+            }
         }
     }
     names(correction) <- names(estpars)[estindex_unique]
-    info <- nameInfoMatrix(info=info, correction=correction, L=L, npars=ncol(L))
-    if(!list$SE) info <- matrix(0, 1, 1)
+    if(list$SE) info <- nameInfoMatrix(info=info, correction=correction, L=L, npars=ncol(L))
+    else info <- matrix(0, 1, 1)
     ret <- list(pars=pars, cycles = cycles - BURNIN - SEMCYCLES, info=if(list$expl) matrix(0) else info,
                 correction=correction, longpars=longpars, converge=converge, SElogLik=0, cand.t.var=cand.t.var, L=L,
-                random=random, lrPars=lrPars, time=c(MH_draws = as.numeric(Draws.time), Mstep=as.numeric(Mstep.time)),
+                random=random, lrPars=lrPars, lr.random=lr.random,
+                time=c(MH_draws = as.numeric(Draws.time), Mstep=as.numeric(Mstep.time)),
                 estindex_unique=estindex_unique, shortpars=longpars[estpars & !redun_constr],
                 fail_invert_info=fail_invert_info, Prior=vector('list', ngroups), collectLL=NaN)
     ret
